#ifndef __force
#define __force

#include <cross.h>
#include <sumstress.h>

/* Computes force applied on nth segment by all other dislocation lines.
 */

template<typename FT> blitz::Array<FT, 1> GetForce(
  const std::vector<StraightSegment3D<FT> >& segments,
  const blitz::Array<FT, 1>& burgers,
  unsigned int n, FT mu, FT nu) {
  /* force */
  blitz::Array<FT, 1> force(3);
  blitz::Array<FT, 2> stress(3, 3);

  stress = SumStresses(segments, burgers, n, mu, nu);
  
  blitz::firstIndex I; blitz::secondIndex J;

  blitz::Array<FT, 1> tmp(3);
  tmp = sum(stress(I, J) * burgers(J), J);
  force = cross(tmp, segments[n].Dir());

  return force;
}

#endif /* __force */
