#ifndef __linearequation
#define __linearequation

#ifdef __linearequation_min
#undef __linearequation_min
#endif
#define __linearequation_min 1e-18

template<typename FT> void LinearEquation(blitz::Array<FT, 2>& equation,
  int cols) {
  int i, j, k, limit = equation.cols() < cols ? equation.cols() : cols;
  blitz::Array<FT, 1> tmp(equation.cols());

  /* the beginning */
  for (i = 0; i < limit; i++) {
    for (j = i; j < equation.rows(); j++) {
      if (fabs(equation(j, i)) < __linearequation_min
        && j + 1 < equation.rows()) { /* move zeroes down */
        tmp = equation(j, blitz::Range::all());
        equation(j, blitz::Range::all()) = equation(j + 1, blitz::Range::all());
        equation(j + 1, blitz::Range::all()) = tmp;
      } 
      if (fabs(equation(j, i)) >= __linearequation_min) { /* check again */
        equation(j, blitz::Range::all()) /= equation(j, i);
      }
    }
    for (j = i + 1; j < equation.rows(); j++) /* subtract line from all below */
      if (fabs(equation(j, i)) > __linearequation_min)
        equation(j, blitz::Range::all()) -= equation(i, blitz::Range::all());
  }
  /* upper triangle matrix */
  for (i = limit - 1; i >= 0; i--) {
    for (j = i - 1; j >= 0; j--)
      for (k = limit; k < equation.cols(); k++) {
        equation(j, k) -= equation(j, i) *
          equation(i, k);
      }
  }
  k = limit < equation.rows() ? limit : equation.rows();
  for (i = 0; i < k; i++)
    for (j = 0; j < k; j++)
      equation(i, j) = i == j;
}

#undef __linearequation_min

#endif /* __linearequation */
