#ifndef __motion
#define __motion

#include <cmath>

#include <cross.h>
#include <force.h>

#ifdef _R_
#undef _R_
#endif
#define _R_ 8.314

#ifdef _k_
#undef _k_
#endif
#define _k_ 1.381e-23

template<class FT> class SegmentMotion3D {
public:
  SegmentMotion3D(FT _D0, FT _Q, FT _T, FT _Omega,
    const blitz::Array<FT, 1>& burgers) {
    FT bb = sum(sqr(burgers));
    b0 = sqrt(bb);
    K = _D0 * exp(-_Q/(_R_*_T)) * _Omega / (bb * _k_ * _T);
    aglide = 10 * K;
  }
  FT AGlide() const { return aglide; }
  FT CClimb(FT be, FT bs) const {
    return K * (be + 10 * bs) / b0; 
  }
private:
  FT b0, K, aglide;
};

#undef _k_
#undef _R_

#endif /* __force */
