#include <iostream>
#include <fstream>
#include <sstream>
#include <vector>
#include <cmath>

#include <blitz/array.h>

#include <linearequation.h>
#include <intersection.h>
#include <dltrans.h>
#include <stress.h>
#include <sumstress.h>
#include <force.h>
#include <motion.h>

using namespace std;
using namespace blitz;

template<typename FT> vector<StraightSegment3D<FT> > CreateLoop(
  FT radius, int segments) {
  vector<StraightSegment3D<FT> > line;
  Array<FT, 1> v(3);
  for (int i = 0; i < segments; i++) {
    v = radius * cos(2 * M_PI * i / segments),
        0,
        radius * sin(2 * M_PI * i / segments);
    line.push_back(v.copy());
  }
  line.push_back(StraightSegment3D<FT>(line[0])); /* uzaven kivka */
  return line;
}

int main() {
  vector<StraightSegment3D<double> > line;
  vector<Array<double, 1> > ncl;
  unsigned int i, j;

  double mu, nu;
  Array<double, 1> burgers(3);
  Array<double, 2> stress(3, 3);
  Array<double, 1> force(3), gforce(3), cforce(3);
  Array<double, 2> base2(3, 3);
  Array<double, 1> tmp(3);
  firstIndex I; secondIndex J;
  double timestep;
  long int N_iterations;

  cout << "m               : ";
  cin >> mu;
  cout << "n               : ";
  cin >> nu;
  cout << "Burgersv vektor : "; 
  cin >> burgers;
  cout << "Dlka kroku      : ";
  cin >> timestep;
  cout << "Poet krok      : ";
  cin >> N_iterations;

  double D0, Q, T, Omega;
  cout << "D0               : ";
  cin >> D0;
  cout << "Q                : ";
  cin >> Q;
  cout << "T                : ";
  cin >> T;
  cout << "Omega            : ";
  cin >> Omega;

  double radius;
  long int segments;
  cout << "Polomr smyky   : ";
  cin >> radius;
  cout << "Poet seek     : ";
  cin >> segments;

  Array<double, 1> be(3), bs(3), nc(3);

  vector<double> spaces;
  vector<double>::const_iterator vdi;
  double space;
  vector<double> forces;
//  vector<Array<double, 1> >::const_iterator vAi;
  
  ofstream output;
  ostringstream name;
  
  SegmentMotion3D<double> segm(D0, Q, T, Omega, burgers);
  
  /* vytvoen dislokan smyky */
  line = CreateLoop<double>(radius, segments);

  long int iterations = 1;
  
  while (iterations <= N_iterations) {
    cout << iterations << ". krok" << endl;
    /* vpoet transforman matice */
    cout << "Zpis souadnic" << endl;
    name.str("");
    name << "segments" << setw(5) << setfill('0') << iterations;
    output.open(name.str().c_str(), ios::binary | ios::out);
    if (output.good()) {
      for (i = 0; i < line.size(); i++)
        output << line[i].Coordinates()(0) << " "
               << line[i].Coordinates()(1) << " "
               << line[i].Coordinates()(2) << endl;
      output.close();
      if (output.bad()) {
        cout << "Chyba - zpis se nezdail." << endl;
      }
    } else {
      cout << "Chyba - nelze vytvoit soubor." << endl;
    }

    if (line[0].Coordinates()(0) == line[line.size()-1].Coordinates()(0) &&
      line[0].Coordinates()(1) == line[line.size()-1].Coordinates()(1) &&
      line[0].Coordinates()(2) == line[line.size()-1].Coordinates()(2)) {
      cout << "Vpoet plochy mnohohelnka: ";
      space = 0;
      for (i = 1; i < line.size(); i++) {
        space += sqrt(sum(sqr(cross(line[i - 1].Coordinates(), line[i].Coordinates()))));
      }
      cout << space << endl;
      spaces.push_back(space);
    }

    cout << "Vpoet transformace" << endl;
    ComputeDLTransformation3D(line, burgers);

    cout << "Pohyb sted" << endl;
    /* Move segment centers according to computed force. */
    ncl.clear();
    forces.clear();
    for (i = 1; i < line.size(); i++) {
      /* Compute force */
      force = GetForce(line, burgers, i, mu, nu);
//      tmp = force / sqrt(sum(sqr(force)));
//      cout << "   >> " << dot(tmp, line[i].Dir()) << endl;
      forces.push_back(sqrt(sum(sqr(force))));
      /* Compute screw and edge components of Burgers vector. */
      bs = dot(burgers, line[i].Dir()) * line[i].Dir();
      be = burgers - bs;
      /* Compute normal vector to the glide plane. */
      tmp = cross(burgers, line[i].Dir());
      if (sum(sqr(tmp)) < 1e-14)
        tmp = cross(line[i].Dir(), line[i - 1].Dir());
      tmp /= sqrt(sum(sqr(tmp)));
      /* Compute glide and climb force */
      cforce = dot(force, tmp) * tmp;
      gforce = force - cforce;
//      cout << i << ". sla: " << force << endl;
/*      nc = timestep * force(0) * segm.AGlide(),
           timestep * force(1) * segm.CClimb(sqrt(sum(sqr(be))), sqrt(sum(sqr(bs)))),
           timestep * force(2) * segm.AGlide();*/
      nc = timestep * (gforce * segm.AGlide() +
        cforce * segm.CClimb(sqrt(sum(sqr(be))), sqrt(sum(sqr(bs)))));
      nc += line[i].Center();
//      cout << i << ". sted: " << nc << endl;
      ncl.push_back(nc.copy());
    }
    cout << "Vpoet souadnic" << endl;
    /* Get new segment coordinates. */
    for (i = 1; i < line.size(); i++) {
      j = i - 1; /* previous segment */
      if (j < 1) j = line.size() - 1;
/*      cout << j << ". : " << endl;
      cout << ncl[j-1] << endl;
      cout << line[j].Dir() << endl;
      cout << ncl[i-1] << endl;
      cout << line[i].Dir() << endl;*/
      line[j].Coordinates() = Intersection(ncl[j-1], line[j].Dir(),
        ncl[i-1], line[i].Dir());
/*      cout << line[j].Coordinates() << endl;*/
    }
    cout << "Zpis sil: " << endl;
    name.str("");
    name << "forces" << setw(5) << setfill('0') << iterations;
    output.open(name.str().c_str(), ios::binary | ios::out);
    if (output.good()) {
      i = 1;
      for (vdi = forces.begin(); vdi != forces.end(); ++vdi)
        output << i++ << " " << *vdi << endl;
      output.close();
      cout << "OK" << endl;
    } else {
      cout << "zpis se nezdail." << endl;
    }
    cout << "Konec kroku" << endl;
    iterations++;
  }

  cout << "Konec vpotu. Nyn budou zapsny plochy do souboru: ";
  output.open("spaces", ios::binary | ios::out);
  if (output.good()) {
    i = 1;
    for (vdi = spaces.begin(); vdi != spaces.end(); ++vdi)
      output << i++ << " " << *vdi << endl;
    output.close();
    cout << "OK" << endl;
  } else {
    cout << "zpis se nezdail." << endl;
  }
   
  return 0;
}

