#include <cmath>
#include <blitz/array.h>

template<class FT> class StressField {
public:
  StressField() : _r(3), _b(3), S(3, 3) { }
  StressField(const blitz::Array<FT, 1>& r_, FT z2_, FT mu_, FT nu_,
    const blitz::Array<FT, 1>& b_) :
    _z2(z2_), _mu(mu_), _nu(nu_),
    _b(b_), _r(r_), S(3, 3) { }
  FT x() const { return _r(0); } FT& x() { return _r(0); }
  FT y() const { return _r(1); } FT& y() { return _r(1); }
  FT z() const { return _r(2); } FT& z() { return _r(2); }
  FT z2() const { return _z2; } FT& z2() { return _z2; }
  FT mu() const { return _mu; } FT& mu() { return _mu; }
  FT nu() const { return _nu; } FT& nu() { return _nu; }
  const blitz::Array<FT, 1>& b() const { return _b; }
  blitz::Array<FT, 1>& b() { return _b; }
  FT XX() const {
    return b()(0) * y() / RRlambda() * xplus();
  }
  FT YY() const {
    return -b()(0) * y() / RRlambda() * yminus();
  }
  FT ZZ() const {
    return b()(0) * (2 * nu() * y() / RRlambda() + y() * lambda() / (R2()*R()));
  }
  FT XY() const {
    return -b()(0) * x() / RRlambda() * yminus();
  }
  FT XZ() const {
    return -b()(0) * x() * y() / (R2()*R())
      + b()(2) * y() * (1 - nu()) / RRlambda();
  }
  FT YZ() const {
    return b()(0) * (nu() / R() - y() * y() / (R2()*R()))
      - b()(2) * x() * (1 - nu()) / RRlambda();
  }
  const blitz::Array<FT, 2>& operator()() {
    S = sigma0() * XX(), sigma0() * XY(), sigma0() * XZ(),
        sigma0() * XY(), sigma0() * YY(), sigma0() * YZ(),
        sigma0() * XZ(), sigma0() * YZ(), sigma0() * ZZ();
    return S;
  }
private:
  FT _z2, _mu, _nu;
  blitz::Array<FT, 1> _b, _r;
  blitz::Array<FT, 2> S;
  
  FT R2() const { return x()*x()+y()*y()+(z2() - z())*(z2() - z()); }
  FT R() const { return sqrt(R2()); }
  FT lambda() const { return z2() - z(); }
  FT RRlambda() const { return R()*(R()+lambda()); }
  FT xplus() const { return 1 + x()*x() / R2() + x()*x() / RRlambda(); }
  FT yminus() const { return 1 - y()*y() / R2() - y()*y() / RRlambda(); }
  FT sigma0() const {
    return mu() / (4 * M_PI * (1 - nu()));
  }
};
