#ifndef __sumstress
#define __sumstress

#include <vector>

/* Computes stress field at 'r' near nth segment of dislocation line. */
template<typename FT> blitz::Array<FT, 2> GetStress(
  const std::vector<StraightSegment3D<FT> >& segments,
  const blitz::Array<FT, 1>& burgers,
  unsigned int n,
  const blitz::Array<FT, 1>& r, FT mu, FT nu) {

  /* stress field */
  blitz::Array<FT, 2> stress(3, 3), stress2(3, 3);

  /* transformations */
  blitz::firstIndex I; blitz::secondIndex J; blitz::thirdIndex K;
  blitz::Array<FT, 1> r_trans(3), tmp(3);
  /* r_i' = T_ji r_j */
  tmp           = r - segments[n].Center();
  r_trans       = sum(segments[n].Base()(J, I) * tmp(J), J);
  /* b_i' = T_ji b_j */
//  burgers_trans = sum(segments[n].Base()(J, I) * burgers(J), J);
  
  /* now we use transformed vectors which are in the segment base */
//  std::cout << "btrans: " << burgers_trans << std::endl;
  
  /* computing stress field */
  StressField<FT> S(r_trans, segments[n].Len()/2, mu, nu, segments[n].Burgers());
  stress = S();
  S.z2() = -segments[n].Len()/2;
  stress -= S();

  /* this stress tensor is also in the segment base, we want to transform it back */
  // obloen
  stress2 = sum(segments[n].Base()(I, K) * stress(K, J), K);
  stress = sum(stress2(I, K) * segments[n].Base()(J, K), K);
  // bn postup
/*  stress2 = sum(stress(K, J) * segments[n].Base()(K, I), K);
  stress = sum(stress2(I, K) * segments[n].Base()(K, J), K);*/
  
  return stress;
}

/* Computes stress field applied on nth segment by all other dislocation lines. */
template<typename FT> blitz::Array<FT, 2> SumStresses(
  const std::vector<StraightSegment3D<FT> >& segments,
  const blitz::Array<FT, 1>& burgers,
  unsigned int n, FT mu, FT nu) {
  unsigned int i;

  /* stress field */
  blitz::Array<FT, 2> stress(3, 3);
  stress = 0; /* this is really important :-) */
  
  for (i = 1; i < segments.size(); i++)
    if (i != n)
      stress += GetStress<FT>(segments, burgers, i, segments[n].Center(), mu, nu);
  
  return stress;
}

#endif
